package com.newriders.jxta.chapter8;

import java.io.FileWriter;
import java.io.IOException;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredTextDocument;

import net.jxta.exception.PeerGroupException;

import net.jxta.id.ID;
import net.jxta.id.IDFactory;

import net.jxta.impl.peergroup.StdPeerGroup;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.pipe.PipeService;

import net.jxta.protocol.PipeAdvertisement;


/**
 * An example to create a set of common Pipe Advertisement to be used 
 * by the PipeServer example application.
 */
public class PipeAdvPopulator
{
    /**
     * The peerGroup for the application.
     */
    private PeerGroup peerGroup = null;


    /**
     * Generates a Pipe Advertisement for the PipeClient/Server example.
     */
    public void generatePipeAdv()
    {
        // Create a new Pipe Advertisement object instance.
        PipeAdvertisement pipeAdv = 
            (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
                PipeAdvertisement.getAdvertisementType());

        // Create a unicast Pipe Advertisement.
        pipeAdv.setName("Chapter 8 Example Unicast Pipe Advertisement");
        pipeAdv.setPipeID((ID) IDFactory.newPipeID(
            peerGroup.getPeerGroupID()));
        pipeAdv.setType(PipeService.UnicastType);
        writePipeAdv(pipeAdv, "UnicastPipeAdv.xml");

        // Create a secure unicast Pipe Advertisement.
        pipeAdv.setName(
            "Chapter 8 Example Secure Unicast Pipe Advertisement");
        pipeAdv.setPipeID((ID) IDFactory.newPipeID(
            peerGroup.getPeerGroupID()));
        pipeAdv.setType(PipeService.UnicastSecureType);
        writePipeAdv(pipeAdv, "SecureUnicastPipeAdv.xml");

        // Create a propagate Pipe Advertisement.
        pipeAdv.setName("Chapter 8 Example Propagate Pipe Advertisement");
        pipeAdv.setPipeID((ID) IDFactory.newPipeID(
            peerGroup.getPeerGroupID()));
        pipeAdv.setType(PipeService.PropagateType);
        writePipeAdv(pipeAdv, "PropagatePipeAdv.xml");
    }

    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform can't 
     *             be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        peerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Runs the application: starts the JXTA platform, generates the Pipe 
     * Advertisements, and stops the JXTA platform.
     *
     * @param   args the command-line arguments passed to the application.
     */
    public static void main(String[] args)
    {
        PipeAdvPopulator p = new PipeAdvPopulator();

        try
        {
            // Initialize the JXTA platform.
            p.initializeJXTA();

            // Generate the Pipe Advertisements to be used by the examples.
            p.generatePipeAdv();

            // Stop the JXTA platform.
            p.uninitializeJXTA();
        }
        catch (PeerGroupException e)
        {
            System.out.println("Error starting JXTA platform: " + e);
            System.exit(1);
        }
    }

    /**
     * Stops the JXTA platform.
     */
    public void uninitializeJXTA()
    {
        // Currently, there isn't any nice way to do this.
        System.exit(0);
    }

    /**
     * Writes the given Pipe Advertisement to a file 
     * with the specified name.
     *
     * @param   pipeAdv the Pipe Advertisement to be written to file.
     * @param   fileName the name of the file to write.
     */
    private void writePipeAdv(PipeAdvertisement pipeAdv, String fileName)
    {
        // Create an XML formatted version of the Pipe Advertisement.
        try
        {
            FileWriter file = new FileWriter(fileName);
            MimeMediaType mimeType = new MimeMediaType("text/xml");
            StructuredTextDocument document = 
                (StructuredTextDocument) pipeAdv.getDocument(mimeType);

            // Output the XML for the advertisement to the file.
            document.sendToWriter(file);
            file.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
}

