package com.newriders.jxta.chapter8;

import java.awt.FlowLayout;
import java.awt.Container;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.net.URL;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;

import net.jxta.endpoint.Message;

import net.jxta.exception.PeerGroupException;

import net.jxta.id.IDFactory;

import net.jxta.peer.PeerID;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.pipe.OutputPipe;
import net.jxta.pipe.OutputPipeEvent;
import net.jxta.pipe.OutputPipeListener;
import net.jxta.pipe.PipeService;

import net.jxta.protocol.PipeAdvertisement;


/**
 * A client application, which sends messages over a pipe to a remote peer.
 */
public class PipeClient implements OutputPipeListener
{
    /**
     * The peerGroup for the application.
     */
    private PeerGroup peerGroup = null;

    /**
     * The pipe to use to send the message to the remote peer.
     */
    private OutputPipe outputPipe = null;

    /**
     * The frame for the user interface.
     */
    private JFrame clientFrame = new JFrame("PipeClient");

    /**
     * The text field in the user interface to accept the message
     * text to be sent over the pipe.
     */
    private JTextField messageText = new JTextField(20);

    /**
     * Indicates whether the pipe has been bound already.
     */
    private boolean initialized = false;

    
    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform can't 
     *             be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        peerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Load the Pipe Advertisement generated by PipeAdvPopulator. This method 
     * tries to create an output pipe that can be used to send messages.
     *
     * @param       fileName the name of the file from which to load 
     *             the Pipe Advertisement.
     * @exception   FileNoteFoundException if the Pipe Advertisement file 
     *             can't be found.
     * @exception   IOException if there is an error binding the pipe.
     */
    public void loadPipeAdv(String fileName) 
        throws FileNotFoundException, IOException
    {
        FileInputStream file = new FileInputStream(fileName);
        MimeMediaType asMimeType = new MimeMediaType("text/xml");

        // Load the advertisement.
        PipeAdvertisement pipeAdv = 
            (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
                 asMimeType, file);

        // Create an output pipe using the advertisement. This version of
        // createOutputPipe uses the PipeClient class as an 
        // OutputPipeListener object.
        PipeService pipeService = peerGroup.getPipeService();
        pipeService.createOutputPipe(pipeAdv, this);
    }

    /**
     * Runs the application: starts the JXTA platform, loads the Pipe 
     * Advertisement from file, and attempts to resolve the pipe.
     *
     * @param   args the command-line arguments passed to the application.
     */
    public static void main(String[] args)
    {
        PipeClient client = new PipeClient();

        if (args.length == 1)
        {
            try
            {
                // Initialize the JXTA platform.
                client.initializeJXTA();

                // Load the Pipe Advertisement.
                client.loadPipeAdv(args[0]);
            }
            catch (PeerGroupException e)
            {
                System.out.println("Error starting JXTA platform: " + e);
                System.exit(1);
            }
            catch (FileNotFoundException e2)
            {
                System.out.println("Unable to load Pipe Advertisement: " 
                    + e2);
                System.exit(1);
            }
            catch (IOException e3)
            {
                System.out.println("Error loading or binding Pipe"
                    + " Advertisement: " + e3);
                System.exit(1);
            }
        }
        else
        {
            System.out.println("You must specify the name of the input"
                + " Pipe Advertisement file.");
        }
    }

    /**
     * The OutputPipeListener event that is triggered when an OutputPipe is
     * resolved by the call to PipeService.createOutputPipe.
     *
     * @param   event the event to use to extract the resolved output pipe.
     */
    public void outputPipeEvent(OutputPipeEvent event)
    {
        // We care about only the first pipe we manage to resolve.
        if (!initialized)
        {
            initialized = true;

            // Get the bound pipe.
            outputPipe = event.getOutputPipe();

            // Show a small GUI to allow the user to send a message.
            showGUI();
        }
    }

    /**
     * Sends a message string to the remote peer using the output pipe.
     *
     * @param   messageString the message text to send to the remote peer.
     */
    private void sendMessage(String messageString)
    {
        PipeService pipeService = peerGroup.getPipeService();
        Message message = pipeService.createMessage();

        // Configure the message object.
        message.setString("MessageText", messageString);

        if (null != outputPipe)
        {
            try
            {
                // Send the message.
                outputPipe.send(message);
            }
            catch (IOException e)
            {
                // Show some warning dialog.
                JOptionPane.showMessageDialog(null, e.toString(), "Error",
                    JOptionPane.WARNING_MESSAGE);
            }
        }
        else
        {
            // Show some warning dialog.
            JOptionPane.showMessageDialog(null, "Output pipe is null!", 
                "Error", JOptionPane.WARNING_MESSAGE);
        }
    }

    /**
     * Configures and displays a simple user interface to allow the user to
     * send text messages. The GUI also allows the user to stop the client
     * application.
     */
    private void showGUI()
    {
        JButton sendButton = new JButton("Send Message");
        JButton quitButton = new JButton("Quit");

        // Populate the GUI frame.
        Container pane = clientFrame.getContentPane();
        pane.setLayout(new FlowLayout());
        pane.add(messageText);
        pane.add(sendButton);
        pane.add(quitButton);

        // Set up listeners for the buttons.
        sendButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e)
                {
                    // Send the message.
                    sendMessage(messageText.getText());

                    // Clear the text.
                    messageText.setText("");
                }
            }
        );
        quitButton.addActionListener(
            new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    clientFrame.hide();
                
                    // Stop the JXTA platform. Currently, there isn't any 
                    // nice way to do this.
                    System.exit(0);
                }
            }
        );

        // Pack and display the user interface.
        clientFrame.pack();
        clientFrame.show();
    }
}