package com.newriders.jxta.chapter8;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Container;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.FileWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.net.URL;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredTextDocument;

import net.jxta.endpoint.Message;

import net.jxta.exception.PeerGroupException;

import net.jxta.impl.util.BidirectionalPipeService;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.pipe.InputPipe;
import net.jxta.pipe.OutputPipe;
import net.jxta.pipe.PipeService;

import net.jxta.protocol.PipeAdvertisement;


/**
 * A simple text messaging application, which uses a bidirectional
 * pipe to send and receive messages.
 */
public class PipeClientServer 
    implements BidirectionalPipeService.MessageListener
{
    /**
     * The peerGroup for the application.
     */
    private PeerGroup peerGroup = null;

    /**
     * The frame for the user interface.
     */
    private JFrame clientFrame = new JFrame("PipeClientServer");

    /**
     * The text field in the user interface to accept the message
     * text to be sent over the pipe.
     */
    private JTextField messageText = new JTextField(20);

    /**
     * A label used to display the received message in the GUI.
     */
    private JLabel receivedText = new JLabel(
        "Waiting to receive a message...");

    /**
     * Indicates whether the pipe has been bound already.
     */
    private boolean initialized = false;

    /**
     * The bidirectional pipe object to use to send and receive messages.
     */
    private BidirectionalPipeService.Pipe pipe = null;


    /**
     * Creates an input pipe and its advertisement using the
     * BidirectionalPipeService. This is used when starting this class up 
     * in "server" mode. The advertisement is saved to file so that another 
     * instance of this class can use the advertisement to start up in 
     * "client" mode.
     */
    public void createPipeAdv() throws IOException
    {
        BidirectionalPipeService pipeService = new 
            BidirectionalPipeService(peerGroup);

        // Create an accept pipe to use to create an input pipe and 
        // listen for connections. "PipeClientServer" is simply the
        // symbolic name that will appear in the Pipe Advertisement
        // created by the BidirectionalPipeService.
        BidirectionalPipeService.AcceptPipe acceptPipe = 
            pipeService.bind("PipeClientServer");

        // Extract the Pipe Advertisement and write it to file.
        PipeAdvertisement pipeAdv = acceptPipe.getAdvertisement();
        try
        {
            FileWriter file = new FileWriter("PipeClientServer.xml");
            MimeMediaType mimeType = new MimeMediaType("text/xml");
            StructuredTextDocument document = 
                (StructuredTextDocument) pipeAdv.getDocument(mimeType);

            // Output the XML for the advertisement to the file.
            document.sendToWriter(file);
            file.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        // "Accept" a connection, meaning set up the input pipe and listen
        // for messages. Set this object as the MessageListener so that
        // we can handle incoming messages without having to spawn a
        // thread to call waitForMessage on the input pipe.
        while (null == pipe)
        {
            try
            {
                pipe = acceptPipe.accept(30000, this);
            }
            catch (InterruptedException e)
            {
                System.out.println("Error trying to accept(): " + e); 
            }
        }

        // Show the user interface.
        showGUI();
    }

    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform can't 
     *              be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        peerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Starts the class in "client" mode, loading a Pipe Advertisement from 
     * the given file. This advertisement is used to create an output pipe 
     * to talk to the remote peer and set up the bidirectional 
     * communications channel.
     *
     * @param       fileName the name of the file from which to load the 
     *             Pipe Advertisement.
     * @exception   FileNoteFoundException if the Pipe Advertisement 
     *             file can't be found.
     * @exception   IOException if there is an error binding the pipe.
     */
    public void loadPipeAdv(String fileName) 
        throws FileNotFoundException, IOException
    {
        FileInputStream file = new FileInputStream(fileName);
        MimeMediaType asMimeType = new MimeMediaType("text/xml");

        // Load the advertisement.
        PipeAdvertisement pipeAdv = 
            (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
                asMimeType, file);

        // Connect using the Pipe Advertisement and the 
        // BidirectionalPipeService.
        BidirectionalPipeService pipeService = 
            new BidirectionalPipeService(peerGroup);

        while (null == pipe)
        {
            try
            {
                System.out.println("Trying...");
                pipe = pipeService.connect(pipeAdv, 30000);
                System.out.println("Done Trying...");
            }
            catch (IOException e)
            {
                // Do nothing.
            }
        }

        // Show the user interface.
        showGUI();

        // There is no way to register a listener with the input pipe used 
        // by the Pipe object to receive message. So, use the 
        // waitForMessage method instead. Not the nicest way to do this, 
        // but it gives you the idea.
        InputPipe input = pipe.getInputPipe();

        while (true)
        {
            try
            {
                Message message = input.waitForMessage();
                
                // Set the user interface to display the message text.
                receivedText.setText(message.getString("MessageText"));
            }
            catch (InterruptedException e)
            {
                // Do nothing, ignore the interruption.
            }
        }
    }

    /**
     * Runs the application. The application can run in either "server" or
     * "client" mode. In "server" mode, the application creates a new Pipe
     * Advertisement, writes it to a file, and binds an input pipe to start
     * listening for incoming messages. In "client" mode, a Pipe 
     * Advertisement is read from a file and used to bind an output pipe to  
     * a remote peer.
     *
     * @param   args the command-line arguments passed to the application. 
     */
    public static void main(String[] args)
    {
        PipeClientServer client = new PipeClientServer();

        if (args.length == 0)
        {
            // No arguments, therefore we must be trying to
            // set up a new server. Create a input pipe and
            // write its advertisement to a file.
            try
            {
                // Initialize the JXTA platform.
                client.initializeJXTA();

                // Create the input connection and save the 
                // Pipe Advertisement.
                client.createPipeAdv();
            }
            catch (PeerGroupException e)
            {
                System.out.println("Error starting JXTA platform: " + e);
                System.exit(1);
            }
            catch (FileNotFoundException e2)
            {
                System.out.println("Unable to load Pipe Advertisement: " 
                    + e2);
                System.exit(1);
            }
            catch (IOException e3)
            {
                System.out.println("Error loading or binding Pipe" 
                    + " Advertisement: " + e3);
                System.exit(1);
            }
        }
        else if (args.length == 1)
        {
            // If there's one argument, then we need to try to
            // connect to an existing server using the Pipe Advertisement
            // in the file specified by the argument.
            try
            {
                // Initialize the JXTA platform.
                client.initializeJXTA();

                // Load the Pipe Advertisement.
                client.loadPipeAdv(args[0]);
            }
            catch (PeerGroupException e)
            {
                System.out.println("Error starting JXTA platform: " + e);
                System.exit(1);
            }
            catch (FileNotFoundException e2)
            {
                System.out.println("Unable to load Pipe Advertisement: " 
                    + e2);
                System.exit(1);
            }
            catch (IOException e3)
            {
                System.out.println("Error loading or binding Pipe" 
                    + " Advertisement: " + e3);
                System.exit(1);
            }
        }
        else
        {
            System.out.println("Usage:");
            System.out.println("'server' mode: PipeClientServer");
            System.out.println("'client' mode: PipeClientServer "
                + "<filename>");

        }
    }

    /**
     * Handles displaying an incoming message to the user interface.
     *
     * @param   message the message received by the input pipe.
     * @param   pipe an OutputPipe to use to send a response.
     */
    public void messageReceived(Message message, OutputPipe pipe)
    {
        // Set the user interface to display the message text.
        receivedText.setText(message.getString("MessageText"));
    }

    /**
     * Sends a message string to the remote peer using the output pipe.
     *
     * @param   messageString the message text to send to the remote peer.
     */
    private void sendMessage(String messageString)
    {
        PipeService pipeService = peerGroup.getPipeService();
        OutputPipe outputPipe = null;

        // Create and configure a message object.
        Message message = pipeService.createMessage();
        message.setString("MessageText", messageString);

        // Get the output pipe from the pipe.
        outputPipe = pipe.getOutputPipe();

        if (null != outputPipe)
        {
            try
            {
                // Send the message.
                outputPipe.send(message);
            }
            catch (IOException e)
            {
                // Show some warning dialog.
                JOptionPane.showMessageDialog(null, e.toString(), "Error",
                   JOptionPane.WARNING_MESSAGE);
            }
        }
        else
        {
            // Show some warning dialog.
            JOptionPane.showMessageDialog(null, "Output pipe is null!", 
                "Error", JOptionPane.WARNING_MESSAGE);
        }
    }

    /**
     * Configures and displays a simple user interface to allow the user to
     * send text messages. The GUI also allows the user to stop the client
     * application.
     */
    private void showGUI()
    {
        JButton sendButton = new JButton("Send Message");
        JButton quitButton = new JButton("Quit");

        JPanel receivePane = new JPanel();
        receivePane.setLayout(new FlowLayout());
        receivePane.add(receivedText);
        
        JPanel sendPane = new JPanel();        
        sendPane.setLayout(new FlowLayout());
        sendPane.add(messageText);
        sendPane.add(sendButton);
        sendPane.add(quitButton);

        // Populate the GUI frame.
        Container pane = clientFrame.getContentPane();
        pane.setLayout(new BorderLayout());
        pane.add(receivePane, BorderLayout.NORTH);
        pane.add(sendPane, BorderLayout.SOUTH);

        // Set up listeners for the buttons.
        sendButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e)
                {
                    // Send the message.
                    sendMessage(messageText.getText());

                    // Clear the text.
                    messageText.setText("");
                }
            }
        );
        quitButton.addActionListener(
            new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    clientFrame.hide();
                
                    // Stop the JXTA platform. Currently, there isn't any 
                    // nice way to do this.
                    System.exit(0);
                }
            }
        );

        // Pack and display the user interface.
        clientFrame.pack();
        clientFrame.show();
    }
}