package com.newriders.jxta.chapter8;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.awt.FlowLayout;
import java.awt.Container;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;

import net.jxta.discovery.DiscoveryService;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;

import net.jxta.endpoint.Message;

import net.jxta.exception.PeerGroupException;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.pipe.InputPipe;
import net.jxta.pipe.PipeMsgEvent;
import net.jxta.pipe.PipeMsgListener;
import net.jxta.pipe.PipeService;

import net.jxta.protocol.PipeAdvertisement;


/**
 * A server application to accept incoming messages on a pipe and display 
 * them to the user.
 */
public class PipeServer implements PipeMsgListener
{
    /**
     * The frame for the user interface.
     */
    private JFrame serverFrame = new JFrame("PipeServer");

    /**
     * A label used to display the received message in the GUI.
     */
    private JLabel messageText = 
        new JLabel("Waiting to receive a message...");
    
    /**
     * The peerGroup for the application.
     */
    private PeerGroup peerGroup = null;

    /**
     * Indicates whether the GUI has been initialized already.
     */
    private boolean initialized = false;

    /**
     * The input pipe used to receive messages.
     */
    private InputPipe inputPipe = null;

        
    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform can't 
     *             be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        peerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Load the Pipe Advertisement generated by PipeAdvPopulator. 
     * This method tries to create an output pipe that can be used 
     * to send messages.
     *
     * @param       fileName the name of the file from which to load 
     *             the Pipe Advertisement.
     * @exception   FileNoteFoundException if the Pipe Advertisement 
     *             file can't be found.
     * @exception   IOException if there is an error binding the pipe.
     */
    public void loadPipeAdv(String fileName) 
        throws FileNotFoundException, IOException
    {
        FileInputStream file = new FileInputStream(fileName);
        MimeMediaType asMimeType = new MimeMediaType("text/xml");

        // Load the advertisement.
        PipeAdvertisement pipeAdv = 
            (PipeAdvertisement) AdvertisementFactory.newAdvertisement(
                asMimeType, file);

        // Publish the discovery to allow peers to find and bind the pipe.
        DiscoveryService discovery = peerGroup.getDiscoveryService();
        discovery.publish(pipeAdv, DiscoveryService.ADV);
        discovery.remotePublish(pipeAdv, DiscoveryService.ADV);
        
        // Create an input pipe using the advertisement.
        PipeService pipeService = peerGroup.getPipeService();
        inputPipe = pipeService.createInputPipe(pipeAdv, this);
    }

    /**
     * Runs the application: starts the JXTA platform, loads the 
     * Pipe Advertisement from file, and creates an input pipe to 
     * use to receive messages.
     *
     * @param   args the command-line arguments passed to the application.
     */
    public static void main(String[] args)
    {
        PipeServer server = new PipeServer();

        if (args.length == 1)
        {
            try
            {
                // Initialize the JXTA platform.
                server.initializeJXTA();

                // Load the Pipe Advertisement.
                server.loadPipeAdv(args[0]);

                // Show the user interface.
                server.showGUI();
            }
            catch (PeerGroupException e)
            {
                System.out.println("Error starting JXTA platform: " + e);
                System.exit(1);
            }
            catch (FileNotFoundException e2)
            {
                System.out.println("Unable to load Pipe Advertisement: " 
                    + e2);
                System.exit(1);
            }
            catch (IOException e3)
            {
                System.out.println("Error loading Pipe Advertisement: " 
                    + e3);
                System.exit(1);
            }
        }
        else
        {
            System.out.println(
                "Specify the name of the input Pipe Advertisement file.");
        }
    }

    /**
     * Handles an incoming message.
     *
     * @param   event the incoming event containing the arriving message.
     */
    public void pipeMsgEvent(PipeMsgEvent event)
    {
        // Extract the message.
        Message message = event.getMessage();

        // Set the user interface to display the message text.
        messageText.setText(message.getString("MessageText"));
    }

    /**
     * Configures and displays a simple user interface to display messages 
     * received by the pipe. The GUI also allows the user to stop the   
     * server application.
     */
    public void showGUI()
    {
        if (!initialized)
        {
            initialized = true;

            JButton quitButton = new JButton("Quit");

            // Populate the GUI frame.
            Container pane = serverFrame.getContentPane();
            pane.setLayout(new FlowLayout());
            pane.add(messageText);
            pane.add(quitButton);

            quitButton.addActionListener(
                new ActionListener()
                {
                    public void actionPerformed(ActionEvent e)
                    {
                        serverFrame.hide();

                        // Stop the JXTA platform. Currently, there isn't 
                        // any nice way to do this.
                        System.exit(0);
                    }
                }
            );

            // Pack and display the user interface.
            serverFrame.pack();
            serverFrame.show();
        }
    }
}