package com.newriders.jxta.chapter11.presence;

import net.jxta.service.Service;


/**
 * An interface for the Presence service, a service that allows peers to
 * exchange presence status information specifying their current status
 * (offline, online, busy, away). This interface defines the operations
 * that a developer can expect to use to manipulate the Presence service, 
 * regardless of which underlying implementation of the service is being 
 * used.
 */
public interface PresenceService extends Service
{
    /**
     * The module class ID for the Presence class of service.
     */
    public static final String refModuleClassID = 
        "urn:jxta:uuid-59A9A948905341119EAB8630EED42AB905";

    /**
     * A status value indicating that a user is currently online but
     * is temporarily away from the device.
     */
    public static final int AWAY= 3;

    /**
     * A status value indicating that a user is currently online but
     * is busy and does not want to be disturbed.
     */
    public static final int BUSY = 2;

    /**
     * A status value indicating that a user is currently offline.
     */
    public static final int OFFLINE = 0;

    /**
     * A status value indicating that a user is currently online.
     */
    public static final int ONLINE = 1;


    /**
     * Add a listener object to the service. When a new Presence Response 
     * Message arrives, the service will notify each registered listener.
     *
     * @param   listener the listener object to register with the service.
     */
    public void addListener(PresenceListener listener);

    /**
     * Announce updated presence information within the peer group.
     *
     * @param   presenceStatus the updated status for the user identified 
     *          by the email address.
     * @param   emailAddress the email address used to identify the user 
     *          associated with the presence info.
     * @param   name a display name for the user associated with the 
     *          presence info.
     */
    public void announcePresence(int presenceStatus, String emailAddress, 
        String name);

    /**
     * Sends a query to find presence information for the user specified
     * by the given email address. Any response received by the service 
     * will be dispatched to registered PresenceListener objects.
     *
     * @param   emailAddress the email address to use to find presence info.
     */
    public void findPresence(String emailAddress);

    /**
     * Removes a given listener object from the service. Once removed, 
     * a listener will no longer be notified when a new Presence Response
     * Message arrives. 
     *
     * @param   listener the listener object to unregister.
     */
    public boolean removeListener(PresenceListener listener);
}