package com.newriders.jxta.chapter11.impl.presence;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import java.util.Enumeration;
import java.util.Vector;

import net.jxta.discovery.DiscoveryEvent;
import net.jxta.discovery.DiscoveryListener;
import net.jxta.discovery.DiscoveryService;

import net.jxta.document.Advertisement;
import net.jxta.document.AdvertisementFactory;
import net.jxta.document.MimeMediaType;

import net.jxta.exception.PeerGroupException;

import net.jxta.id.ID;

import net.jxta.impl.protocol.DiscoveryResponse;

import net.jxta.peergroup.PeerGroup;

import net.jxta.protocol.DiscoveryResponseMsg;
import net.jxta.protocol.ModuleImplAdvertisement;

import net.jxta.service.Service;

import com.newriders.jxta.chapter11.impl.protocol.PresenceAdv;

import com.newriders.jxta.chapter11.presence.PresenceListener;
import com.newriders.jxta.chapter11.presence.PresenceService;

import com.newriders.jxta.chapter11.protocol.PresenceAdvertisement;


/**
 * The implementation of the PresenceService interface. This service
 * builds on top of the Discovery service to provide the functionality 
 * for requesting and providing presence information.
 */
public class PresenceServiceImpl implements PresenceService,
    DiscoveryListener
{
    /**
     * The Module Specification ID for the Presence service.
     */
    public static final String refModuleSpecID = 
        "urn:jxta:uuid-59A9A948905341119EAB8630EED42AB"
        + "9F4611FF6377C4931AE71BE299B9F34DF06";

    /**
     * The default expiration timeout for published presence advertisements.
     * Set to 1 minute.
     */
    private static final int DEFAULT_EXPIRATION = 1000 * 60 * 1;
    
    /**
     * The default lifetime for published presence advertisements.
     * Set to 1 minutes.
     */
    private static final int DEFAULT_LIFETIME = 1000 * 60 * 5;
    
    /**
     * The element name for the presence advertisement's email address info.
     */
    private static final String tagEmailAddress = "EmailAddress";

    /**
     * The Discovery service used to publish presence information.
     */
    private DiscoveryService discovery = null;

    /**
     * The Module Implementation advertisement for this service.
     */
    private Advertisement implAdvertisement = null;

    /**
     * The local Peer ID.
     */
    private String localPeerID = null;
    
    /**
     * The peer group to which the service belongs.
     */
    private PeerGroup peerGroup = null;

    /**
     * A unique query ID that can be used to track a query.
     */
    private int queryID = 0;
    
    /**
     * The set of listener objects registered with the service.
     */
    private Vector registeredListeners = new Vector();


    /**
     * PresenceServiceImpl constructor comment.
     */
    public PresenceServiceImpl()
    {
        super();
    }

    /**
     * Add a listener object to the service. When a new Presence Response 
     * Message arrives, the service will notify each registered listener.
     * This method is synchronized to prevent multiple threads from 
     * altering the set of registered listeners simultaneously.
     *
     * @param   listener the listener object to register with the service.
     */
    public synchronized void addListener(PresenceListener listener)
    {
        registeredListeners.addElement(listener);
    }

    /**
     * Announce presence status information to the peer group.
     *
     * @param   presenceStatus the current status to announce.
     * @param   emailAddress the user's email address.
     * @param   name the user's display name.
     */
    public void announcePresence(int presenceStatus, String emailAddress, 
        String name)
    {
        if (discovery != null)
        {
            /*
            PresenceAdvertisement presenceInfo = (PresenceAdvertisement) 
                AdvertisementFactory.newAdvertisement(
                    PresenceAdvertisement.getAdvertisementType());
            */
            // In some earlier versions of JXTA, registering the 
            // advertisement doesn't work properly. To work around this, 
            // simply instantiate the advertisement implementation directly.
            // This is not the recommended way to get an advertisement. 
            // The recommended way is shown in the commented line
            // preceeding this comment.
            PresenceAdvertisement presenceInfo = new PresenceAdv();

            // Configure the new advertisement.
            presenceInfo.setPresenceStatus(presenceStatus);
            presenceInfo.setEmailAddress(emailAddress);
            presenceInfo.setName(name);
            presenceInfo.setPeerID(localPeerID);

            try
            {
                // Publish the advertisement locally.
                discovery.publish(presenceInfo, DiscoveryService.ADV, 
                    DEFAULT_EXPIRATION, DEFAULT_LIFETIME);
            }
            catch (IOException e)
            {
                System.out.println("Error publishing locally: " + e);
            }
            
            // Publish the advertisement remotely.
            discovery.remotePublish(presenceInfo, DiscoveryService.ADV, 
                DEFAULT_LIFETIME);
        }
    }

    /**
     * Handle notification of arriving discovery response messages, 
     * determine whether the response contains presence information, 
     * and, if so, dispatch the presence information to registered
     * PresenceListeners.
     *
     * @param   event the object containing the discovery response.
     */
    public void discoveryEvent(DiscoveryEvent event)
    {
        DiscoveryResponseMsg response = event.getResponse();

        // Extract the PresenceAdvertisement from the response.
        Enumeration responses = response.getResponses();
        while (responses.hasMoreElements())
        {
            String responseElement = (String) responses.nextElement();
            
            // Check for null response advertisement.
            if (null != responseElement)
            {
                // Parse the advertisement.
                try
                {
                    ByteArrayInputStream stream = 
                        new ByteArrayInputStream(
                            responseElement.getBytes());
                    
                    /*
                    PresenceAdvertisement advertisement = 
                        (PresenceAdvertisement) 
                            AdvertisementFactory.newAdvertisement(
                                new MimeMediaType("text/xml"), stream);
                    */

                    // In some earlier versions of JXTA, registering the 
                    // advertisement doesn't work properly. To work around 
                    // this, simply instantiate the advertisement 
                    // implementation directly. This is not the recommended 
                    // way to get an advertisement. The recommended way 
                    // is shown in the commented line preceeding this
                    // comment.

                    PresenceAdvertisement advertisement = 
                        new PresenceAdv(stream);

                    // Dispatch the advertisement to the registered presence
                    // listeners.
                    Enumeration listeners = registeredListeners.elements();
                    while (listeners.hasMoreElements())
                    {
                        PresenceListener listener = 
                            (PresenceListener) listeners.nextElement();

                        // Notify the listener of the presence update.
                        listener.presenceUpdated(advertisement);
                    }
                }
                catch (IOException e)
                {
                    // Obviously not a response to our query for presence
                    // information. Ignore the error.
                    System.out.println("Error in discoveryEvent: " + e);
                }
            }
            else
            {
                System.out.println("Response advertisement is null!");
            }
        }
    }

    /**
     * Sends a query to find presence information for the user specified
     * by the given email address. Any response received by the service 
     * will be dispatched to registered PresenceListener objects.
     *
     * @param   emailAddress the email address to use to find presence info.
     */
    public void findPresence(String emailAddress)
    {
        // Make sure the service has been started.
        if (discovery != null)
        {
            // Send a remote discovery for presence information.
            discovery.getRemoteAdvertisements(null, DiscoveryService.ADV,
                tagEmailAddress, emailAddress, 0, null);

            // Do a local discovery for presence information.
            try
            {
                Enumeration enum = discovery.getLocalAdvertisements(
                    DiscoveryService.ADV, tagEmailAddress, emailAddress);

                while (enum.hasMoreElements())
                {
                    PresenceAdvertisement advertisement = 
                        (PresenceAdvertisement) enum.nextElement();

                    // Dispatch the advertisement to the registered presence
                    // listeners.
                    Enumeration listeners = registeredListeners.elements();
                    while (listeners.hasMoreElements())
                    {
                        PresenceListener listener = 
                            (PresenceListener) listeners.nextElement();

                        // Notify the listener of the presence update.
                        listener.presenceUpdated(advertisement);
                    }
                }
            }
            catch (IOException e)
            {
                System.out.println("Error in findPresence: " + e);
            }
        }
    }

    /**
     * Returns the advertisement for this service. In this case, this is 
     * the ModuleImplAdvertisement passed in when the service was 
     * initialized.
     *
     * @return  the advertisement describing this service.
     */
    public Advertisement getImplAdvertisement()
    {
        return implAdvertisement;
    }

    /**
     * Returns an interface used to protect this service.
     *
     * @return  the wrapper object to use to manipulate this service.
     */
    public Service getInterface()
    {
        // We don't really need to provide an interface object to protect
        // this service, so this method simply returns the service itself.
        return this;
    }

    /**
     * Initialize the service.
     *
     * @param       group the PeerGroup containing this service.
     * @param       assignedID the identifier for this service.
     * @param       implAdv the advertisement specifying this service.
     * @exception   PeerGroupException is not thrown ever by this
     *              implementation.
     */
    public void init(PeerGroup group, ID assignedID, Advertisement implAdv)
        throws PeerGroupException
    {
        // Save the module's implementation advertisement.
        implAdvertisement = (ModuleImplAdvertisement) implAdv;

        // Save a reference to the group of which that this service is 
        // a part.
        peerGroup = group;

        // Get the local Peer ID.
        localPeerID = group.getPeerID().toString();
        
        // Register the advertisement type.
        // In some earlier versions of JXTA, registering the advertisement
        // doesn't work properly. To work around this, you can instead 
        // simply instantiate the advertisement implementation directly. 
        // This is not the recommended way to get an advertisement.
        // In this class, I've used the workaround in the discoveryEvent
        // and accouncePresence methods, but I've provided the 
        // as well.
        /*
        AdvertisementFactory.registerAdvertisementInstance(
            PresenceAdvertisement.getAdvertisementType(),
                new PresenceAdv.Instantiator());
        */
    }

    /**
     * Remove a given listener object from the service. Once removed, 
     * a listener will no longer be notified when a new Presence Response
     * Message arrives. This method is synchronized to prevent multiple 
     * threads from altering the set of registered listeners simultaneously.
     *
     * @param   listener the listener object to unregister.
     */
    public synchronized boolean removeListener(PresenceListener listener)
    {
        return registeredListeners.removeElement(listener);
    }

    /**
     * Start the service.
     *
     * @param   args the arguments to the service. Not used.
     * @return  0 to indicate the service started.
     */
    public int startApp(String[] args)
    {
        // Now that the service is being started, set the DiscoveryService
        // object to use to publish presence information.
        discovery = peerGroup.getDiscoveryService();

        // Add ourselves as a listener.
        discovery.addDiscoveryListener(this);
 
        return 0;
    }

    /**
     * Stop the service.
     */
    public void stopApp()
    {
        if (discovery != null)
        {
            // Unregister ourselves as a listener.
            discovery.removeDiscoveryListener(this);
            discovery = null;

            // Empty the set of listeners.
            registeredListeners.removeAllElements();
        }
    }
}